/********************************************************
   Paper Bit - 8 Step Sequenzer
   for an Atmega328

   by Wolfgang Spahn   25Sept2016

 *********************************************************/


// Maximum of the step poti (use 5 digit less then you measured)
int StepMax = 335; //312; //335; //490; //330;

int StepLEDs[8] =  {18, 13, 11, 10, 7, 5, 3, 4};  // all 8 LED pins
int StepButtons[8] = {19, 17, 12, 9, 8, 1, 2, 0}; // all 8 button pins
int CV_Values[8] = {0, 0, 0, 0, 0, 0, 0, 0};      // Voltage output
float CV_Groove[8] = {1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0};    // length of each note

// add your grooves here - length of each note:
float CV_Groove0[8] = {1.3, 0.7, 0.7, 0.7, 1.3, 0.7, 0.7, 0.7};
int CV_Values0[8] = {200, 175, 150, 125, 200, 125, 100, 150};
float CV_Groove1[8] = {0.3, 1, 1, 1.7, 0.3, 1, 1, 1.7};
int CV_Values1[8] = {200, 100, 225, 150, 175, 100, 175, 150};
float CV_Groove2[8] = {1, 0.7, 1, 0.7, 1.5, 0.5, 0.5, 1.5};
int CV_Values2[8] = {40, 50, 0, 40, 0, 50, 70, 0};
float CV_Groove3[8] = {1, 1, 1, 1, 1, 1, 1, 1};
int CV_Values3[8] = {30, 60, 120, 240, 120, 60, 30, 0};
float CV_Groove4[8] = {1, 1, 0.3, 0.7, 1, 1, 0.3, 0.7};
float CV_Groove5[8] = {0.5, 1.0, 1.5, 1.0, 0.5, 1.0, 1.5, 1.0};
int CV_Values5[8] = {75, 81, 87, 91, 95, 91, 87, 81};
float CV_Groove6[8] = {0.5, 1.0, 1.5, 1.0, 0.5, 1.0, 1.5, 1.0};
int CV_Values6[8] = {85, 91, 97, 101, 105, 101, 97, 91};
float CV_Groove7[8] = {0.5, 1.0, 1.5, 1.0, 0.5, 1.0, 1.5, 1.0};
int CV_Values7[8] = {95, 101, 107, 111, 115, 111, 107, 101};

int StepsPotiPin = A0;
int SpeedPotiPin = A1;
int CVPotiPin = A2;
int CVPin = 6;
int CV_Out = 0;

unsigned long DelayValue = 0;
int CV_Value;
int StepsPoti = 1023;
int Steps = 8;
int StepButton = 0;
int StepNumber = 0;

unsigned long previousMillis = 0;
unsigned long currentMillis = 0;
int LEDState = LOW;

void setup() {
  // sets the spead on pin6 (just for a Atmega328 chip):
  TCCR0B = TCCR0B & 0b11111000 | 0x01;

  pinMode(CVPin, OUTPUT);
  for (int i = 0; i < 8; i++) {
    pinMode(StepLEDs[i], OUTPUT);
  }
  for (int i = 0; i < 8; i++) {
    pinMode(StepButtons[i], INPUT_PULLUP);
  }
  TestLEDs();
}

void loop() {
  if (Steps > 1) {
    for (int i = 0; i < Steps; i++) {
      LEDState = HIGH;
      digitalWrite(StepLEDs[i], LEDState);
      analogWrite(CVPin, CV_Values[i]);
      while (LEDState == HIGH) {
        SwitchCheck();
        DelayValue = analogRead(SpeedPotiPin);
        DelayValue = map(DelayValue, 1023, 0, 5, 100000);
        DelayValue = DelayValue * CV_Groove[i];
        currentMillis = millis();
        if (currentMillis - previousMillis >= DelayValue) {
          previousMillis = currentMillis;
          LEDState = LOW;
          digitalWrite(StepLEDs[i], LEDState);
        }
      }
    }
  }
  if (Steps == 1) {
    StepNumber = analogRead(SpeedPotiPin);
    StepNumber = map(StepNumber, 1023, 0, 7, 0);
    LEDState = HIGH;
    digitalWrite(StepLEDs[StepNumber], LEDState);
    analogWrite(CVPin, CV_Values[StepNumber]);
    ButtonCheck();
    SwitchCheck();
    LEDState = LOW;
    digitalWrite(StepLEDs[StepNumber], LEDState);
  }
}

// checks if the switch or the poti is on
void SwitchCheck() {
  StepsPoti = analogRead(StepsPotiPin);
  if (StepsPoti < 500) {
    Steps = map(StepsPoti, 0, StepMax, 1, 8);
    ButtonCheck();
  }
  else if (StepsPoti > StepMax + 100) { //535
    SelectPreset();
  }
}



// checks if a button is pressed and assign the value for the CV
void ButtonCheck() {
  for (int j = 0; j < 8; j++) {
    StepButton = digitalRead(StepButtons[j]);
    if (StepButton == LOW) {
      CV_Value = analogRead(CVPotiPin);
      CV_Value = map(CV_Value, 0, 1023, 0, 255);
      CV_Values[j] = CV_Value;
    }
  }
}

// assignes the groove
void SelectPreset() {
  for (int j = 0; j < 8; j++) {
    StepButton = digitalRead(StepButtons[j]);
    if (StepButton == LOW) {
      switch (j) {
        case 0:
          for (int i = 0; i < 8; i++) {
            CV_Groove[i] = CV_Groove0[i];
            CV_Values[i] = CV_Values0[i];
          }
          break;
        case 1:
          for (int i = 0; i < 8; i++) {
            CV_Groove[i] = CV_Groove1[i];
            CV_Values[i] = CV_Values1[i];
          }
          break;
        case 2:
          for (int i = 0; i < 8; i++) {
            CV_Groove[i] = CV_Groove2[i];
            CV_Values[i] = CV_Values2[i];
          }
          break;
        case 3:
          for (int i = 0; i < 8; i++) {
            CV_Groove[i] = CV_Groove3[i];
            CV_Values[i] = CV_Values3[i];
          }
          break;
        case 4:
          for (int i = 0; i < 8; i++) {
            CV_Groove [i] = CV_Groove4 [i];
          }
          break;
        case 5:
          for (int i = 0; i < 8; i++) {
            CV_Groove [i] = CV_Groove5 [i];
            CV_Values[i] = CV_Values5[i];
          }
          break;
        case 6:
          for (int i = 0; i < 8; i++) {
            CV_Groove [i] = CV_Groove6 [i];
            CV_Values[i] = CV_Values6[i];
          }
          break;
        case 7:
          for (int i = 0; i < 8; i++) {
            CV_Groove [i] = CV_Groove7 [i];
            CV_Values[i] = CV_Values7[i];
          }
          break;
      }
    }
  }
}

/////////////////////////////////////////////////////////////////

// testing the LEDs
void TestLEDs() {
  for (int i = 0; i < 8; i++) {
    digitalWrite(StepLEDs[i], HIGH);
  }
  delay(10000);
  for (int i = 0; i < 8; i++) {
    digitalWrite(StepLEDs[i], LOW);
  }
  delay(10000);
}

// testing the Buttons
void Buttontest() {
  for (int j = 0; j < 8; j++) {
    StepButton = digitalRead(StepButtons[j]);
    if (StepButton == LOW) {
      digitalWrite(StepLEDs[j], HIGH);
      delay(100);
    }
    for (int k = 0; k < 8; k++) {
      digitalWrite(StepLEDs[k], LOW);
    }
  }
}

// turns the LEDs on
void SwitchLED(int LEDnumber) {
  LEDnumber = LEDnumber - 1;
  if (LEDnumber < 0) {
    LEDnumber = 7;
  }
  digitalWrite(StepLEDs[LEDnumber], LEDState);

}


